# -*- coding: utf-8 -*-

# Scrapy settings for Obsidian project
#
# For simplicity, this file contains only settings considered important or
# commonly used. You can find more settings consulting the documentation:
#
#     http://doc.scrapy.org/en/latest/topics/settings.html
#     http://scrapy.readthedocs.org/en/latest/topics/downloader-middleware.html
#     http://scrapy.readthedocs.org/en/latest/topics/spider-middleware.html

BOT_NAME = 'Obsidian'

SPIDER_MODULES = ['Obsidian.spiders']
NEWSPIDER_MODULE = 'Obsidian.spiders'


# Crawl responsibly by identifying yourself (and your website) on the user-agent
#USER_AGENT = 'Obsidian (+http://www.yourdomain.com)'

# Obey robots.txt rules
ROBOTSTXT_OBEY = True

# Configure maximum concurrent requests performed by Scrapy (default: 16)
#CONCURRENT_REQUESTS = 32

# Configure a delay for requests for the same website (default: 0)
# See http://scrapy.readthedocs.org/en/latest/topics/settings.html#download-delay
# See also autothrottle settings and docs
#DOWNLOAD_DELAY = 3
# The download delay setting will honor only one of:
#CONCURRENT_REQUESTS_PER_DOMAIN = 16
#CONCURRENT_REQUESTS_PER_IP = 16

# Disable cookies (enabled by default)
#COOKIES_ENABLED = False

# Disable Telnet Console (enabled by default)
#TELNETCONSOLE_ENABLED = False

# Override the default request headers:
#DEFAULT_REQUEST_HEADERS = {
#   'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
#   'Accept-Language': 'en',
#}

# Enable or disable spider middlewares
# See http://scrapy.readthedocs.org/en/latest/topics/spider-middleware.html
#SPIDER_MIDDLEWARES = {
#    'Obsidian.middlewares.MyCustomSpiderMiddleware': 543,
#}

# Enable or disable downloader middlewares
# See http://scrapy.readthedocs.org/en/latest/topics/downloader-middleware.html
#DOWNLOADER_MIDDLEWARES = {
#    'Obsidian.middlewares.MyCustomDownloaderMiddleware': 543,
#}

# Enable or disable extensions
# See http://scrapy.readthedocs.org/en/latest/topics/extensions.html
#EXTENSIONS = {
#    'scrapy.extensions.telnet.TelnetConsole': None,
#}

# Configure item pipelines
# See http://scrapy.readthedocs.org/en/latest/topics/item-pipeline.html
#ITEM_PIPELINES = {
#    'Obsidian.pipelines.SomePipeline': 300,
#}

# Enable and configure the AutoThrottle extension (disabled by default)
# See http://doc.scrapy.org/en/latest/topics/autothrottle.html
#AUTOTHROTTLE_ENABLED = True
# The initial download delay
#AUTOTHROTTLE_START_DELAY = 5
# The maximum download delay to be set in case of high latencies
#AUTOTHROTTLE_MAX_DELAY = 60
# The average number of requests Scrapy should be sending in parallel to
# each remote server
#AUTOTHROTTLE_TARGET_CONCURRENCY = 1.0
# Enable showing throttling stats for every response received:
#AUTOTHROTTLE_DEBUG = False

# Enable and configure HTTP caching (disabled by default)
# See http://scrapy.readthedocs.org/en/latest/topics/downloader-middleware.html#httpcache-middleware-settings
#HTTPCACHE_ENABLED = True
#HTTPCACHE_EXPIRATION_SECS = 0
#HTTPCACHE_DIR = 'httpcache'
#HTTPCACHE_IGNORE_HTTP_CODES = []
#HTTPCACHE_STORAGE = 'scrapy.extensions.httpcache.FilesystemCacheStorage'



#!/usr/bin/env python
# coding=utf-8
# File Name: myspider.py
# Author: amoblin <amoblin@gmail.com>
# Created Time: 五  8/ 5 18:12:46 2016

from Obsidian.ddspider import DDSpider
from Obsidian.ddsourcejsonconfig import DDSourceJsonConfig

class JsonSpider(DDSpider):
    config = DDSourceJsonConfig("config.json")

#    name = config.name
    name = "jsonspider"
    allowed_domains= config.allowed_domains
    start_urls = config.start_urls
    prefix = config.prefix
    link_array_pipline = config.link_array_pipline
    main_content_pipline = config.main_content_pipline
    item_pipline = config.item_pipline




#!/usr/bin/env python
# -*- coding:utf-8 -*-
# file name: obsidian.py
# description: TODO
# create date: 2016-09-19 13:44:31
# author: amoblin
# This file is created by Marboo<http://marboo.io> template file $MARBOO_HOME/.media/starts/default.py
# 本文件由 Marboo<http://marboo.io> 模板文件 $MARBOO_HOME/.media/starts/default.py 创建

from Obsidian.cmdline import execute

execute()



#!/usr/bin/env python
# coding=utf-8
# File Name: ddsourcejsonconfig.py
# Author: amoblin <amoblin@gmail.com>
# Created Time: 五  8/26 14:01:27 2016

import json

from Obsidian.ddsourceconfig import DDSourceConfig

class DDSourceJsonConfig(DDSourceConfig):
    def __init__(self, filename=""):
        if len(filename) == 0:
            return
        with open(filename) as data_file:
            config = json.load(data_file)
            self.name = config["name"]
            self.allowed_domains= config["allowed_domains"]
            self.start_urls = config["start_urls"]
            self.prefix = config["prefix"]
            self.link_array_pipline = config["link_array_pipline"]
            self.main_content_pipline = config["main_content_pipline"]
            self.item_pipline = config["item_pipline"]
            self.entry = config["entry"]



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# file name: cmdline.py
# description: TODO
# create date: 2016-09-19 14:43:48
# author: amoblin
# This file is created by Marboo<http://marboo.io> template file $MARBOO_HOME/.media/starts/default.py
# 本文件由 Marboo<http://marboo.io> 模板文件 $MARBOO_HOME/.media/starts/default.py 创建

import os
import sys
import scrapy.cmdline

base = os.path.dirname(os.path.abspath(os.path.realpath(__file__)))

def execute():
    if len(sys.argv) <= 1:
        print("usage: %s [config file]" % sys.argv[0])
        sys.exit(0)
    filename = sys.argv[1]

    output = os.path.join(os.getcwd(), "obsidian_output.json")
    if len(sys.argv) > 2:
        output = sys.argv[2]

    if not output.startswith('/'):
        output = os.path.join(os.getcwd(), output)

    if not filename.startswith('/'):
        filename = os.path.join(os.getcwd(), filename)

    if not os.path.isfile(filename):
        print("file not exist: %s" % sys.argv[1])
        sys.exit(0)

    dest = os.path.join(base, "spiders/config.json")
    if os.path.islink(dest):
        os.remove(dest)
    os.symlink(filename, dest)
    os.chdir(os.path.join(base, "spiders"))
    init_settings()
    scrapy.cmdline.execute(("scrapy crawl jsonspider -o %s" % output).split())

def init_settings():
    content = """
[settings]
default = Obsidian.settings

[deploy]
#url = http://localhost:6800/
project = Obsidian
"""
    cfg_file = os.path.join(os.path.dirname(base), "scrapy.cfg")
    if not os.path.isfile(cfg_file):
        f = open(cfg_file, 'w')
        f.write(content)

if __name__ == "__main__":
    execute()



#!/usr/bin/env python
# coding=utf-8
# File Name: ddsourceconfig.py
# Author: amoblin <amoblin@gmail.com>
# Created Time: 五  8/26 13:53:17 2016

class DDSourceConfig(object):
    name = ""
    prefix = ""
    allowed_domains = []
    start_urls = []
    link_array_pipline = [{}]
    main_content_pipline = [{}]
    item_pipline = {}

    def __init__(self):
        pass



#!/usr/bin/env python
# coding=utf-8
# File Name: dd_spider.py
# Author: amoblin <amoblin@gmail.com>
# Created Time: 一  8/ 8 13:51:08 2016

import time
import scrapy
from Obsidian.items import ObsidianItem

from scrapy import Spider
from scrapy.http import Request

class DDSpider(Spider):

    def juice(self, s, pipline, should_extract=1):
        selector = s
        command = "selector"
        for pip in pipline:
            if not 'type' in pip:
                continue
            if pip["type"] != "re":
                command = "%s.%s('%s')" % (command, pip["type"], pip["value"])
            else:
                command = "%s.re(%s)" % (command, pip["value"])

        print("*"*20)
        print(command)
        print("*"*20)
        selector = self.eval(selector, command)

        if len(pipline) > 0 and pipline[-1]["type"] != "re" and should_extract:
            selector = selector.extract()

            if not 'should_join' in pipline[-1] or pipline[-1]['should_join'] != 0:
                return "".join(selector).strip()
        return selector

    def drink(self, content, regex='', index=1):
        return "".join(content).strip()

    def eval(self, selector, code):
#        print "********** will execute: %s" % code
        return eval(code)


    def parse(self, response):
        if len(self.link_array_pipline) == 0:
            yield self.parse_page(response)
        elif isinstance(self.link_array_pipline[0], list):
            for item in self.parse_step_generator()(response):
                yield item
        else:
            for item in map(lambda x: self.prefix + x, self.juice(response, self.link_array_pipline)):
                yield Request(url = item, callback=self.parse_page)

    def parse_step_generator(self, i=0):
        def fun(response):
            if i >= len(self.link_array_pipline):
                yield self.parse_page(response)
            else:
                pipline = self.link_array_pipline[i]
                for item in map(lambda x: self.prefix + x, self.juice(response, pipline)):
                    yield Request(url = item, callback=self.parse_step_generator(i+1))
        return fun

    def parse_page(self, response):
        main_content_selector = self.juice(response, self.main_content_pipline, 0)
        item = ObsidianItem()
        for key, value in self.item_pipline.items():
            if key not in ObsidianItem.fields:
                ObsidianItem.fields[key] = scrapy.Field()
            item[key] = self.juice(main_content_selector, value)
        item['url'] = response.url
        item['crawl_time'] = int(time.time())
        item['status'] = 1
        return item



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name = 'Obsidian',
    version = '1.0.1',
    packages = find_packages(),
    scripts = ['obsidian.py'],

    # Project uses reStructuredText, so ensure that the docutils get
    # installed or upgraded on the target machine
    install_requires = [
        'scrapy',
    ],

    package_data = {
        # If any package contains *.txt or *.rst files, include them:
        '': ['*.txt', '*.rst', '*.md'],
    },

    # metadata for upload to PyPI
    author = "amoblin",
    author_email = "amoblin@gmail.com",
    description = "Obsidian make web crawl easier",
    license = "MIT",
    keywords = "web crawl json",
    url = "https://github.com/amoblin/Obsidian",

    # could also include long_description, download_url, classifiers, etc.
)
